package Message::Passing::Exception::ConnectionDisconnected;
use Moo;
use Data::Dumper ();
use namespace::clean -except => 'meta';

with 'Message::Passing::Exception';

1;

=head1 NAME

Message::Passing::Exception::ConnectionDisconnected - A connection disconnected

=head1 ATTRIBUTES

=head1 AUTHOR, COPYRIGHT AND LICENSE

See L<Message::Passing>.

=cut

